/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "TreeWalker.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomDOMObject.h"
#include "gdom/GdomNode.h"
#include "gdom/GdomTreeWalker.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomNode.h"
#include "gdom/PrivateGdomTreeWalker.h"
#include "gdom/PrivateIntGdomNode.h"
#include "gdom/PrivateIntGdomTreeWalker.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
gpointer toGDOM(WebCore::TreeWalker* obj)
{
    return WebCore::cacheDOMObject<WebCore::TreeWalker, GDOMTreeWalker>(obj);
}
} /* namespace WebCore */
extern "C" {

GDOMTreeWalker::GDOMTreeWalker(PassRefPtr<WebCore::TreeWalker> impl)
    : GDOMObject()
    , m_impl(impl)
{
}

GDOMTreeWalker::~GDOMTreeWalker()
{
    m_impl.get()->deref();
    WebCore::GDOMScriptInterpreter::forgetDOMObject(m_impl.get());

}


/* XXX TODO: custom function gdom_tree_walker_parent_node */


/* XXX TODO: custom function gdom_tree_walker_first_child */


/* XXX TODO: custom function gdom_tree_walker_last_child */


/* XXX TODO: custom function gdom_tree_walker_previous_sibling */


/* XXX TODO: custom function gdom_tree_walker_next_sibling */


/* XXX TODO: custom function gdom_tree_walker_previous_node */


/* XXX TODO: custom function gdom_tree_walker_next_node */

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "TreeWalker.h"
#include <gdom/GdomTreeWalker.h>
#include <gdom/PrivateGdomTreeWalker.h>
#include <gdom/PrivateIntGdomTreeWalker.h>
#include <gdom/GdomTreeWalkerprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomTreeWalker, gdom_tree_walker, GDOM_TYPE_DOM_OBJECT);

/*
struct _GdomTreeWalkerPrivate {
    class WebCore::TreeWalker *item;
};

#define GDOM_TREE_WALKER_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_TREE_WALKER, GdomTreeWalkerPrivate))
*/
G_END_DECLS
WebCore::TreeWalker *
gdom_tree_walker_get_private(GdomTreeWalker *request)
{
    GDOMTreeWalker *res;
    res = static_cast<GDOMTreeWalker*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::TreeWalker*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_ROOT,
    PROP_WHAT_TO_SHOW,
    PROP_FILTER,
    PROP_EXPAND_ENTITY_REFERENCES,
    PROP_CURRENT_NODE,
};


static void gdom_tree_walker_finalize(GObject* object)
{
    /*GdomTreeWalker* request = GDOM_TREE_WALKER(object);*/
    /*GdomTreeWalkerPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_DOM_OBJECT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_tree_walker_parent_class)->finalize(object);
}

static void gdom_tree_walker__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomTreeWalker *self = GDOM_TREE_WALKER(object);
    WebCore::TreeWalker * gself = (gdom_tree_walker_get_private(self));
    switch (prop_id) {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_tree_walker__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomTreeWalker *self = GDOM_TREE_WALKER(object);
    WebCore::TreeWalker * gself = (gdom_tree_walker_get_private(self));
    switch (prop_id) {
    case PROP_ROOT:
    {
         PassRefPtr<WebCore::Node> ptr = gself->root();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_WHAT_TO_SHOW:
    {
         g_value_set_ulong (value, gself->whatToShow());
         break;
    }
    case PROP_FILTER:
    {
         PassRefPtr<WebCore::NodeFilter> ptr = gself->filter();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_EXPAND_ENTITY_REFERENCES:
    {
         g_value_set_boolean (value, gself->expandEntityReferences());
         break;
    }
    case PROP_CURRENT_NODE:
    {
         PassRefPtr<WebCore::Node> ptr = gself->currentNode();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_tree_walker_class_init(GdomTreeWalkerClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_tree_walker_finalize;
    g_klass->set_property = gdom_tree_walker__set_property_fn;
    g_klass->get_property = gdom_tree_walker__get_property_fn;

    g_object_class_install_property (g_klass, PROP_ROOT,
            g_param_spec_object ("root", /* name */
                "tree_walker_root", /* short description */
                "read-only  GdomNode * TreeWalker.root", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_NODE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_WHAT_TO_SHOW,
            g_param_spec_ulong ("what_to_show", /* name */
                "tree_walker_what_to_show", /* short description */
                "read-only  gulong TreeWalker.what_to_show", /* longer - could do with some extra doc stuff here */
                 0,        /* min */
                G_MAXULONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_FILTER,
            g_param_spec_object ("filter", /* name */
                "tree_walker_filter", /* short description */
                "read-only  GdomNodeFilter * TreeWalker.filter", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_NODE_FILTER, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_EXPAND_ENTITY_REFERENCES,
            g_param_spec_boolean ("expand_entity_references", /* name */
                "tree_walker_expand_entity_references", /* short description */
                "read-only  gboolean TreeWalker.expand_entity_references", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_CURRENT_NODE,
            g_param_spec_object ("current_node", /* name */
                "tree_walker_current_node", /* short description */
                "read-write  GdomNode * TreeWalker.current_node", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_NODE, /* gobject type */
                 (GParamFlags)G_PARAM_READWRITE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomTreeWalkerPrivate));*/
}

static void gdom_tree_walker_init(GdomTreeWalker* request)
{
    /*GdomTreeWalkerPrivate* priv = GDOM_TREE_WALKER_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

